;; sieve.el
;; Copyright  NAGY Andras <nagya@inf.elte.hu>

(defun sieve-string-list (list)
  (concat "[\"" (mapconcat 'identity list "\", \"") "\"]"))

(defun sieve-test-list (list)
  (concat "(" (mapconcat 'sieve-test list ", ") ")"))

(defun sieve-test-word (word)
  (cond
   ((symbolp word)
    (symbol-name word))

   ((stringp word)
    (concat "\"" word "\""))

   ((and (listp word)
	 (stringp (car word)))
    (sieve-string-list word))

   ((and (listp word)
	 (listp (car word)))
    (sieve-test-list word))))

(defun sieve-test (test)
  (mapconcat 'sieve-test-word test " "))

(defun sieve-script (&optional method crosspost)
  (let* ((newsrc (cdr gnus-newsrc-alist))
	 script)
    (dolist (info newsrc)
      (when (or (not method) (equal (gnus-info-method info) method))
	(let*
	    ((group (gnus-info-group info))
	     (spec (gnus-group-find-parameter group 'sieve t)))
	  (when spec
	    (push (concat "if " (sieve-test spec) " {\n\t"
			  "fileinto \""
			  (gnus-group-real-name group) "\";\n}")
		  script)))))

    (mapconcat 'identity script (if crosspost "\n" "\nels"))))

(defconst sieve-region-start "\n## Begin Gnus Sieve Script\n")
(defconst sieve-region-end   "\n## End Gnus Sieve Script\n")

(defun sieve-update (file)
  (interactive "fSieve script: ")
  (find-file file)
  (beginning-of-buffer)
  (re-search-forward
   (concat (regexp-quote sieve-region-start) "\\(.\\|\n\\)*"
	   (regexp-quote sieve-region-end)))
  ;;(replace-match
  ;; (concat sieve-region-start (sieve-script) sieve-region-end) t t)
  (delete-region (match-beginning 0) (match-end 0))
  (insert-string (concat sieve-region-start (sieve-script) sieve-region-end))
  (save-buffer))

(provide 'sieve)
